@@ -13,8 +13,11 @@
 #include <jg_jnu.h>
 #include <gtk_java.h>
 
+#include "org_gnu_gconf_ConfClient.h"
+
 #ifndef _Included_org_gnu_gconf_ConfClient
 #define _Included_org_gnu_gconf_ConfClient
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -77,7 +80,7 @@ typedef struct
 	jstring nameSpace;
 } NotifyData;
 
-void notifyCallback(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer data)
+static void notifyCallback(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer data)
 {
 	NotifyData *nd = (NotifyData*)data;
 	jobject entry_g = getStructHandle(nd->env, entry,
@@ -85,7 +88,7 @@ void notifyCallback(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpoin
 	(*nd->env)->CallVoidMethod(nd->env, nd->obj, nd->methodID, entry_g, nd->nameSpace);
 }
 
-void notifyFree(gpointer data)
+static void notifyFree(gpointer data)
 {
 	NotifyData *nd = (NotifyData*)data;
 	(*nd->env)->DeleteGlobalRef(nd->env, nd->obj);
@@ -486,7 +489,7 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 	}
 	int i = 0;
 	jclass aCls = NULL;
-	guint len;
+
 	jobjectArray ar;
 	if (NULL == val)
 		return NULL;
@@ -514,7 +517,7 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
+			obj = (*env)->NewObject(env, aCls, mid, (jint)GPOINTER_TO_INT(item->data));
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		}
 		else if (vt == GCONF_VALUE_FLOAT) {
@@ -522,7 +525,7 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jdouble*)item->data);
+			obj = (*env)->NewObject(env, aCls, mid, (jdouble)(*(gdouble*)(item->data)));
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		}
 		else if (vt == GCONF_VALUE_BOOL) {
@@ -530,11 +533,12 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jboolean)((gboolean)item->data));
+			obj = (*env)->NewObject(env, aCls, mid, (jboolean)(GPOINTER_TO_INT(item->data) != 0));
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		}
 		else if (vt == GCONF_VALUE_SCHEMA) {
-			jmethodID mid = (*env)->GetStaticMethodID(env, aCls, "getConfSchema", "(I)V");
+		  //			jmethodID mid = (*env)->GetStaticMethodID(env, aCls, "getConfSchema", "(I)V");
+			jmethodID mid = (*env)->GetStaticMethodID(env, aCls, "getConfSchema", "(org/gnu/glib/Handle)org/gnu/glib/Handle");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
@@ -722,13 +726,13 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1lis
 			data = (gpointer)&value;
 		}
 		else if (vt == GCONF_VALUE_SCHEMA) {
-			jint value;
+			jobject value;
 			aCls = (*env)->FindClass(env, "org/gnu/gconf/ConfSchema");
-			fid = (*env)->GetFieldID(env, aCls, "handle", "I");
+			fid = (*env)->GetFieldID(env, aCls, "handle", "org/gnu/glib/Handle");
 			if (fid == NULL)
 				return (jboolean)JNI_FALSE;
-			value = (jint)(*env)->GetObjectField(env, obj, fid);
-			data = (gpointer)value;
+			value = (*env)->GetObjectField(env, obj, fid);
+			data = (gpointer)getPointerFromHandle(env, value);
 		}
 		gslist = g_slist_append(gslist, data);
 	}
@@ -746,13 +750,16 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1lis
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1pair
   (JNIEnv *env, jclass cls, jobject client, jstring key, jint carType, jint
-  		cdrType, jint car, jint cdr, jobject exception)
+  		cdrType, jobject car, jobject cdr, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = NULL;
 	gboolean ret = gconf_client_set_pair(client_g, str, (GConfValueType)carType, 
-			(GConfValueType)cdrType, (gconstpointer)car, (gconstpointer)cdr, &err);
+					     (GConfValueType)cdrType,
+					     (gconstpointer)getPointerFromHandle(env,car), 
+					     (gconstpointer)getPointerFromHandle(env,cdr), 
+					     &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	if (err != NULL)	{
 		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
